import processing.serial.*;
Serial port;

// Optional brightness multiplier if you aren't managing this in touch.
float maxBright = 1.0;

// Define the width and height of your video stream coming in through Spout
int rows = 480;
int columns = 8;

// Just a multiplier for number of channels per led. (R-G-B) Probably wont need to touch this..
int dataChunkSize = 3;

// Byte array that will hold the values we pull from the canvas after a SPOUT update. This byte array is sent in WHOLE to the led's via serial at the end of the draw loop.
byte[] vals = new byte[rows * columns * dataChunkSize];

//PGraphics canvas;
Spout client;
PImage img;

// This group of variables deals with reporting the frame rate every so often. Change "frameTimeReportIters" to report more often, or less. setting this too frequently will affect performance.
int frameTimeReportIters = 30;
int frameTimeReportCounter = 0;
int frameTimeCumulativeHolder = 0;
int obligatoryDelay = 2;


// ------------- SETUP
public void setup() {
  
  // Some basic init stuff..
  size(rows, columns, P2D);
  img = createImage(width, height, RGB);
  background(0);
  
  // Init Spout client receiver ..
  client = new Spout();
  client.initReceiver("", img);
  
  // Declare Com port as string name, and setup the port.
  String portName = "COM5";
  println(portName);
  port = new Serial(this, portName, 115200);
  delay(500);
}


// ------------- DRAW
void draw() {
  
  // for frame rate reporting use.
  int timeCapture0 = millis();
  
  // pulls most recent frame from Spout, and sets it to the processing image.
  img = client.receiveTexture(img);
  image(img, 0, 0, width, height);
  
  // Loads the most recent pixels after the spout update.
  loadPixels();
  
  // Loop through our total pixel count in processing.. (should match total count we're sending via processing)
  for(int pix = 0; pix < width*height; pix++){
    
    // Grab color
    color c = pixels[pix];
    
    // Convert / split color to RGB values.
    int r = (int) (((c >> 16) & 0xFF) * maxBright);
    int g = (int) (((c >> 8) & 0xFF) * maxBright);
    int b = (int) ((c & 0xFF) * maxBright);
    
    // Set RGB values into vals byte array.
    vals[pix*dataChunkSize] = (byte) r;
    vals[pix*dataChunkSize+1] = (byte) g;
    vals[pix*dataChunkSize+2] = (byte) b;
  }
  
  // We're finished with the for loop and have our updated pixel values in "val". Sending over serial Port to leds!
  port.write(vals);
  
  // Ending frame rate reporting timing variable..
  int timeCapture1 = millis();
  
  
  // All this nonsense below is just for printing framerate. Probably could have been done better but got lazy.
  frameTimeCumulativeHolder += (timeCapture1 - timeCapture0);
  if(frameTimeReportCounter <= 0){
    frameTimeReportCounter = frameTimeReportIters;
    print("FPS...: ");
    if((frameTimeCumulativeHolder / frameTimeReportIters)-obligatoryDelay != 0){
      println(((frameTimeCumulativeHolder / frameTimeReportIters)-obligatoryDelay));
      println("");
      frameTimeCumulativeHolder = 0;
    }
  }
frameTimeReportCounter -= 1;

  
  
}
